/*************************************************************************************\


 #     #   #####   #     #   #####    #####   ######      #  #######  #####   ######
 #     #  #     #  ##    #  #     #  #     #  #     #    #      #    #     #  #     #
 #  #  #  #     #  # #   #  #              #  #     #   #       #          #  #     #
 #  #  #  #######  #  #  #  # #####   #####   #     #  #   #    #     #####   #     #
 #  #  #  #     #  #   # #  #     #        #  #     #  #######  #          #  #     #
 #  #  #  #     #  #    ##  #     #  #     #  #     #      #    #    #     #  #     #      
  ## ##   #     #  #     #   #####    #####   ######       #    #     #####   ######


                                  Preprocessor of
                 3D Wang Tiles with Ellipsoidal Inclusions for T3D
                           Copyright: Daniel Rypl 2018

\*************************************************************************************/

#define REUSE_POINTS           /* more efficient */


static void
process_aa_side_geometry(void)
{
	ellipse_rec *ellipse = NULL;
	point_rec point, point1, point2, point3, point4, point5, point6, center;
	point_rec *pnt = NULL, *pnt1 = NULL, *pnt2 = NULL, *pnt3 = NULL, *pnt4 = NULL;
	vector_rec vec, vec_max, vec_min;
	is_point_rec *is_point = NULL, *is_point1 = NULL, *is_point2 = NULL, *is_point3 = NULL, *is_point4 = NULL;
	double weight, coord, size;
	long bg_vertex_id, bg_curve_id, bg_curve_id_minus, bg_curve_id_plus;
	long ver_id, ver_id1, ver_id2, ver_id3, ver_id4, ver_id5, ver_id6, ver_id7, ver_id8, ver_id9, ver_id10;
	long cur_id, cur_id1, cur_id2, ptch_id;
	logic output_patch, output_side_vertex;
	logic output_bb_edge_vertex, output_cc_edge_vertex;
	logic output_bb_edge_curve, output_cc_edge_curve;
	logic halving_bb, halving_cc, cross_halving;

	aa_intersection = aa_touching = NO;

	if((aa_face_is = inclusion -> face_is[AA]) == NULL)return;

#ifdef DEBUG_PRINT
	fprintf(stderr, "Processing %s-side geometry\n", string_aa);

	fprintf(stderr, "aa = %s\n", string_aa);
	fprintf(stderr, "bb = %s\n", string_bb);
	fprintf(stderr, "cc = %s\n\n", string_cc);
#endif

	bb_edge_is = inclusion -> edge_is[BB];
	cc_edge_is = inclusion -> edge_is[CC];

	output_patch = NO;
	output_side_vertex = NO;
	output_bb_edge_vertex = NO;
	output_cc_edge_vertex = NO;
	output_bb_edge_curve = NO;
	output_cc_edge_curve = NO;

	ellipse = &(aa_face_is -> ellipse);
	add_vec(center, ellipse -> center, shift_vec);            /* shift ellipse center */
					
/* IMPORTANT: do not return after processing particular branch of code;
	 there are checks at the end of the routine !!! */

	if(ellipse -> max == 0.0){                 /* side touching */
		aa_touching = YES;

		if(bb_count == 0 && cc_count == 0){        /* not touching edge */
			if((aa_face_is -> side[AABB] & ref_position[bb_ind]) != 0 && (aa_face_is -> side[AACC] & ref_position[cc_ind]) != 0){
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, center.x, center.y, center.z, tile_id);

				aa_side_vertex_array[aa_ind][aa_side_vertices[aa_ind]++] = vertex_id;
				output_side_vertex = YES;

				if(aa_ind == ind_aa)side_vertex_array[side_vertices++] = vertex_id;                  /* touching from inside */
			}
		}
		else{                                    /* touching edge */
			if(cc_count == 0){                      /* touching bb-edge */
#ifdef DEBUG
				if(bb_count != 1)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
					is_point = &(bb_edge_is -> point[0]);
					ver_id = is_point -> id;
					
					if(ver_id == 0){
						point.aa = par_corner[aa_ind].aa;
						point.bb = is_point -> data.coord + shift_bb;
						point.cc = par_corner[cc_ind].cc;
						
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);
						
						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point -> id = ver_id;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point -> data);
						output_bb_edge_vertex = YES;
						bb_edge_curves_2[bbbb_ind]++;
					}
				}
			}
			if(bb_count == 0){                      /* touching cc-edge */
#ifdef DEBUG
				if(cc_count != 1)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
				if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
					is_point = &(cc_edge_is -> point[0]);
					ver_id = is_point -> id;
						
					if(ver_id == 0){
						point.aa = par_corner[aa_ind].aa;
						point.bb = par_corner[bb_ind].bb;
						point.cc = is_point -> data.coord + shift_cc;
						
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);
						
						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point -> id = ver_id;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point -> data);
						output_cc_edge_vertex = YES;
						cc_edge_curves_2[cccc_ind]++;
					}
				}
			}
#if 0
			if(bb_count == 1 && cc_count == 1){      /* touching corner */
			}
#endif
		}
	}
	else{                                      /* side intersection */
		aa_intersection = YES;

#ifdef USE_SIDE
		sprintf(side_spec, "side %d", 2 * AA + aa_ind + 1);
#endif

		*mirror_spec = '\0';
		if(inclusion -> mat != 0){
			if(tile.code[2 * AA + aa_ind] != 0){
				code = &code_array[tile.code[2 * AA + aa_ind] - 1];

/* CAUTION: when processing tile using a particular code for the first time on both opposite sides, special care must be taken;
	 normally, intersection of inclusion with the side of that tile, which uses the code of that side for the first time, is mirrored
	 by intersections of all other instances of that inclusion with sides having the same code;
	 but this is not possible if processed tile uses particular code for the first time on both opposite sides and if the primary instance 
	 of the inclusion is intersecting positive side;
	 the reason is that in this particular case code structure stores as side of the first occurrence the negative side, which is consequence 
	 of ordering in which tile sides are processed (negative side always preceeds opposite positive side) when storing the first use of the code;
	 thus when processing the primary instance, the intersection with negative side is not available (the patch does not exist yet) 
	 because the secondary instance, which intersects in this case the negative side (which is the stored first time side), was not processed yet, 
	 which is the consequence of the fact that secondary instances are processed after the primary instance;
	 therefore the mirroring of the intersection of the primary instance with positive tile side by the intersection of the secondary instance 
	 with negative side must be used in this particular case !!!
	 in other words, when processing the primary instance, no mirroring is applied;
	 when processing the secondary instance, mirror of the already established intersesection of the primary instance with the positive side is applied;
	 this should have no impact on ordering of nodes on related sides !!!

   XN           XP    code(XN) = code(XP)
   |            |

	 o------------o     p - primary instance
   |            |     s - secondary instance
   s            p
   s   <-----   p     <--- direction of mirroring 
   s            p
   |            |
   |            |
   o------------o */

				if(code -> tile_id == tile_id){
					if(code -> id == tile.code[2 * AA + 1 - aa_ind]){
						if(aa_ind != ind_aa){                /* <==> per_aa == 1 <==> secondary instance of inclusion */
#ifdef HALF_MAX_TILES
						  //ptch_id = ((aa_side_patches[aa_ind] + 1) * max_tiles + tile_id) * 10 + 2 * AA + 1 - aa_ind + 1;     /* keep compatible with get_aa_side_patch_id macro */
							ptch_id = ((aa_side_patches[aa_ind] + 1) * max_tiles + tile_id) * 10 + 2 * AA + ind_aa + 1;         /* keep compatible with get_aa_side_patch_id macro */
#else
						  //ptch_id = ((aa_side_patches_total[aa_ind] + 1) * max_tiles + tile_id) * 10 + 2 * AA + 1 - aa_ind + 1;     /* keep compatible with get_aa_side_patch_id macro */
							ptch_id = ((aa_side_patches_total[aa_ind] + 1) * max_tiles + tile_id) * 10 + 2 * AA + ind_aa + 1;         /* keep compatible with get_aa_side_patch_id macro */
#endif
#ifdef USE_SIMPLE
							sprintf(mirror_spec, "mirror %ld simple", ptch_id);
#else
							sprintf(mirror_spec, "mirror %ld", ptch_id);
#endif
						}
					}
				}
				else{
#ifdef HALF_MAX_TILES
					ptch_id = ((aa_side_patches[aa_ind] + 1) * max_tiles + code -> tile_id) * 10 + code -> side + 1;    /* keep compatible with get_aa_side_patch_id macro */
#else
					ptch_id = ((aa_side_patches_total[aa_ind] + 1) * max_tiles + code -> tile_id) * 10 + code -> side + 1;    /* keep compatible with get_aa_side_patch_id macro */
#endif
#ifdef USE_SIMPLE
					sprintf(mirror_spec, "mirror %ld simple", ptch_id);
#else
					sprintf(mirror_spec, "mirror %ld", ptch_id);
#endif
				}
			}
		}
	
		if(bb_count == 0 && cc_count == 0){        /* not crossing edges */
			if((aa_face_is -> side[AABB] & ref_position[bb_ind]) != 0 && (aa_face_is -> side[AACC] & ref_position[cc_ind]) != 0){
				copy_mul_vec(vec_max, ellipse -> major, ellipse -> max);
				copy_mul_vec(vec_min, ellipse -> minor, 2.0 * ellipse -> min);

/* I do rely on fact that center is already shifted */

				add_vec(point1, center, vec_max);
				sub_vec(point2, center, vec_max);
				add_vec(point3, point1, vec_min);
				add_vec(point4, point2, vec_min);
				sub_vec(point5, point1, vec_min);
				sub_vec(point6, point2, vec_min);
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
				write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id1 = ++curve_id, ver_id1, ver_id2, CURVE_RATE, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
				write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);

				aa_patch_id = get_aa_side_patch_id(aa_ind);
				write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

				aa_side_curve_array[aa_side_curves++] = cur_id1;
				aa_side_curve_array[aa_side_curves++] = cur_id2;

				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
		}
		else if(bb_count == 1 && cc_count == 0){   /* touching bb-edge */
			if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
				if((aa_face_is -> side[AACC] & ref_position[cc_ind]) != 0){
					pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);            /* cc- */
					pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);            /* cc+ */
					
					size = fabs(pnt1 -> cc - pnt2 -> cc);
					size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
					
					add_vec(point1, *pnt1, shift_vec);
					add_vec(point2, *pnt2, shift_vec);
					copy_vec(point3, point1);
					copy_vec(point4, point2);
#ifndef REUSE_POINTS
					copy_vec(point5, point1);
					copy_vec(point6, point2);
#endif
					is_point = &(bb_edge_is -> point[0]);
					ver_id = is_point -> id;
					
					if(ver_id == 0){
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
						
						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point -> id = (cc_ind == 0) ? ver_id1 : ver_id2;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point -> data);
						output_bb_edge_vertex = YES;
						bb_edge_curves_2[bbbb_ind]++;
					}
					else{
						if(cc_ind == 0){
							ver_id1 = ver_id;
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
						}
						else{
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
							ver_id2 = ver_id;
						}
					}
					
					point3.bb += 4.0 * size;
					point4.bb += 4.0 * size;
					write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id1 = ++curve_id, ver_id1, ver_id2, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
#ifndef REUSE_POINTS
					point5.bb -= 4.0 * size;
					point6.bb -= 4.0 * size;
					write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
					point3.bb -= 8.0 * size;
					point4.bb -= 8.0 * size;
					write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
					aa_patch_id = get_aa_side_patch_id(aa_ind);
					write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

					aa_side_curve_array[aa_side_curves++] = cur_id1;
					aa_side_curve_array[aa_side_curves++] = cur_id2;

					aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
					aa_side_patches_total[aa_ind]++;
#endif
					output_patch = YES;
				}
				else{
					is_point = &(bb_edge_is -> point[0]);
					ver_id = is_point -> id;
					
					if(ver_id == 0){
						pnt = &(aa_face_is -> extreme_pnt[2 * AACC + 1 - cc_ind]);          /* cc- or cc+ on bb-edge */
						add_vec(point, *pnt, shift_vec);

						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);

						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point -> id = ver_id;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point -> data);
						output_bb_edge_vertex = YES;
						bb_edge_curves_2[bbbb_ind]++;
					}
				}
			}
		}
		else if(bb_count == 0 && cc_count == 1){   /* touching cc-edge */
			if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
				if((aa_face_is -> side[AABB] & ref_position[bb_ind]) != 0){
					pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);            /* bb- */
					pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);            /* bb+ */
						
					size = fabs(pnt1 -> bb - pnt2 -> bb);
					size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
					
					add_vec(point1, *pnt1, shift_vec);
					add_vec(point2, *pnt2, shift_vec);
					copy_vec(point3, point1);
					copy_vec(point4, point2);
#ifndef REUSE_POINTS
					copy_vec(point5, point1);
					copy_vec(point6, point2);
#endif
					is_point = &(cc_edge_is -> point[0]);
					ver_id = is_point -> id;
					
					if(ver_id == 0){
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
						
						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point -> id = (bb_ind == 0) ? ver_id1 : ver_id2;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point -> data);
						output_cc_edge_vertex = YES;
						cc_edge_curves_2[cccc_ind]++;
					}
					else{
						if(bb_ind == 0){
							ver_id1 = ver_id;
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
						}
						else{
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
							ver_id2 = ver_id;
						}
					}
					
					point3.cc -= 4.0 * size;
					point4.cc -= 4.0 * size;
					write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id1 = ++curve_id, ver_id1, ver_id2, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
#ifndef REUSE_POINTS
					point5.cc += 4.0 * size;
					point6.cc += 4.0 * size;
					write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
					point3.cc += 8.0 * size;
					point4.cc += 8.0 * size;
					write_record("curve %ld vertex %ld %ld order 4 rate %g density 2 tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
					aa_patch_id = get_aa_side_patch_id(aa_ind);
					write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

					aa_side_curve_array[aa_side_curves++] = cur_id1;
					aa_side_curve_array[aa_side_curves++] = cur_id2;

					aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
					aa_side_patches_total[aa_ind]++;
#endif
					output_patch = YES;
				}
				else{
					is_point = &(cc_edge_is -> point[0]);
					ver_id = is_point -> id;
						
					if(ver_id == 0){
						pnt = &(aa_face_is -> extreme_pnt[2 * AABB + 1 - bb_ind]);          /* bb- or bb+ on cc-edge */
						add_vec(point, *pnt, shift_vec);
						
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);
							
						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point -> id = ver_id;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point -> data);
						output_cc_edge_vertex = YES;
						cc_edge_curves_2[cccc_ind]++;
					}
				}
			}
		}
		else if(bb_count == 1 && cc_count == 1){   /* touching both bb-edge and cc-edge */
			if((bb_edge_is -> side & ref_position[bb_ind]) != 0 && (cc_edge_is -> side & ref_position[cc_ind]) != 0){
				pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);     /* cc- or cc+ on bb-edge */
				pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);     /* bb- or bb+ on cc-edge */
					
				add_vec(point1, *pnt1, shift_vec);
				add_vec(point2, *pnt2, shift_vec);
				
				point.aa = center.aa;
				point.bb = point2.bb;
				point.cc = point1.cc;
				
				weight = aa_face_is -> weight;
				
				is_point1 = &(bb_edge_is -> point[0]);
				ver_id1 = is_point1 -> id;
				
				is_point2 = &(cc_edge_is -> point[0]);
				ver_id2 = is_point2 -> id;
				
				if(ver_id1 == 0){
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
					
					bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
					bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
					output_bb_edge_vertex = YES;
					bb_edge_curves_2[bbbb_ind]++;
				}
				if(ver_id2 == 0){
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
					
					cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point2 -> id = ver_id2;
					cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point2 -> data);
					output_cc_edge_vertex = YES;
					cc_edge_curves_2[cccc_ind]++;
				}
				
				if(bb_ind == cc_ind ){
					weight = sqrt(1.0 - weight * weight);
					
					write_record("curve %ld vertex %ld %ld order 3 rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, weight);
					write_record("curve %ld vertex %ld %ld order 3 rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id1, ver_id2, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
				}
				else{
					write_record("curve %ld vertex %ld %ld order 3 rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id1, ver_id2, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, weight);
					write_record("curve %ld vertex %ld %ld order 3 rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id1, CURVE_RATE, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
				}
				
				aa_patch_id = get_aa_side_patch_id(aa_ind);
				write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

				aa_side_curve_array[aa_side_curves++] = cur_id1;
				aa_side_curve_array[aa_side_curves++] = cur_id2;

				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
			else{
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
					is_point1 = &(bb_edge_is -> point[0]);
					ver_id1 = is_point1 -> id;
					
					if(ver_id1 == 0){
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 1 - cc_ind]);          /* cc- or cc+ on bb-edge */
						add_vec(point1, *pnt1, shift_vec);
						
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
						
						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
						output_bb_edge_vertex = YES;
						bb_edge_curves_2[bbbb_ind]++;
					}
				}
				if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
					is_point2 = &(cc_edge_is -> point[0]);
					ver_id2 = is_point2 -> id;
					
					if(ver_id2 == 0){
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1 - bb_ind]);          /* bb- or bb+ on cc-edge */
						add_vec(point2, *pnt2, shift_vec);
						
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
						
						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point2 -> id = ver_id2;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point2 -> data);
						output_cc_edge_vertex = YES;
						cc_edge_curves_2[cccc_ind]++;
					}
				}
			}
		}
		else if(bb_count == 2 && cc_count == 0){                       /* crossing bb-edge */
			if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
				aa_patch_id = get_aa_side_patch_id(aa_ind);

				pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);         /* bb- */
				pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);         /* bb+ */

				is_point1 = &(bb_edge_is -> point[0]);
				ver_id1 = is_point1 -> id;

				is_point2 = &(bb_edge_is -> point[1]);
				ver_id2 = is_point2 -> id;

				coord = tile.corner[ind_cc].cc;  /* caution: ind_cc is correct !!! */

				point1.aa = point2.aa = par_corner[aa_ind].aa;
				point1.cc = point2.cc = par_corner[cc_ind].cc;

				point1.bb = is_point1 -> data.coord + shift_bb; 
				point2.bb = is_point2 -> data.coord + shift_bb; 
					
				if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
					if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);

					bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
					bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
					bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point2 -> id = ver_id2;
					bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point2 -> data);
					output_bb_edge_vertex = YES;
					bb_edge_curves_2[bbbb_ind]++;
				}

				if(bb_curve_id == 0){
					write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);

					bb_edge_curve_1_array[bbbb_ind][bb_edge_curves_1[bbbb_ind]++] = bb_curve_id;
					output_bb_edge_curve = YES;
				}

				if(cc_ind == 1)bb_curve_id = -bb_curve_id;

				halving_bb = YES;
				if(aa_face_is -> edge_half[AABB] == NO){
					if(cc_ind == 0){
						if(pnt1 -> cc <= coord || pnt2 -> cc <= coord){    /* CAUTION: floating point number based decision */
							halving_bb = NO;

							weight = aa_face_is -> weight;
							if(pnt1 -> cc <= pnt2 -> cc){
								pnt3 = pnt1;
								weight = sqrt(1.0 - weight * weight);
							}
							else
								pnt3 = pnt2;
							pnt4 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);
						}
					}
					else{
						if(pnt1 -> cc >= coord || pnt2 -> cc >= coord){    /* CAUTION: floating point number based decision */
							halving_bb = NO;

							weight = aa_face_is -> weight;
							if(pnt1 -> cc <= pnt2 -> cc){
								pnt3 = pnt2;
								weight = sqrt(1.0 - weight * weight);
							}
							else
								pnt3 = pnt1;
							pnt4 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);
						}
					}
				}

				if(halving_bb == NO){
					add_vec(point3, *pnt3, shift_vec);
					add_vec(point4, *pnt4, shift_vec);

					point.aa = center.aa;
					point.bb = point3.bb;
					point.cc = point4.cc;
					
					write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point3.x, point3.y, point3.z);
					write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point4.x, point4.y, point4.z);
					write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, vertex_id - 1, vertex_id, CURVE_RATE);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
											
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

					if(cc_ind == 0)
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
					else
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
											
					write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

					aa_side_curve_array[aa_side_curves++] = curve_id;
				}
				else{
#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to %s\n\n", string_bb);
#endif
					pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
					pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
									
					size = fabs(pnt1 -> cc - pnt2 -> cc);
					size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
					if((cc_ind == 0 && center.cc > par_corner[0].cc) || (cc_ind == 1 && center.cc < par_corner[1].cc)){          /* CAUTION */
#else
						if(center.cc * sign[cc_ind] > par_corner[cc_ind].cc * sign[cc_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to bb-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
#ifndef REUSE_POINTS
							copy_vec(point5, point1);
							copy_vec(point6, point2);
#endif		
							if(cc_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
							}
							else{
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
							}
							
							point3.bb += 4.0 * size;
							point4.bb += 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
							
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
							point5.bb -= 4.0 * size;
							point6.bb -= 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
							point3.bb -= 8.0 * size;
							point4.bb -= 8.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
								
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = cur_id1;
							aa_side_curve_array[aa_side_curves++] = cur_id2;
						}
						else{ 

/* halving parallel to bb-edge */

							point1.aa = point2.aa = par_corner[aa_ind].aa;
							point1.cc = point2.cc = center.cc;
							
							point1.bb = center.bb - 2.0 * size;
							point2.bb = center.bb + 2.0 * size;
							
							if(cc_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
							
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
							
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
							
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
							
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}

				if(cc_ind == 1)bb_curve_id = -bb_curve_id;

				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
		}
		else if(bb_count == 0 && cc_count == 2){                          /* crossing cc-edge */
			if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
				aa_patch_id = get_aa_side_patch_id(aa_ind);

				pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);            /* cc- */
				pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);            /* cc+ */

				is_point1 = &(cc_edge_is -> point[0]);
				ver_id1 = is_point1 -> id;

				is_point2 = &(cc_edge_is -> point[1]);
				ver_id2 = is_point2 -> id;

				coord = tile.corner[ind_bb].bb;  /* caution: ind_bb is correct !!! */

				point1.aa = point2.aa = par_corner[aa_ind].aa;
				point1.bb = point2.bb = par_corner[bb_ind].bb;

				point1.cc = is_point1 -> data.coord + shift_cc; 
				point2.cc = is_point2 -> data.coord + shift_cc; 
					
				if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
					if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);

					cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point1 -> id = ver_id1;
					cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point1 -> data);
					cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point2 -> id = ver_id2;
					cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point2 -> data);
					output_cc_edge_vertex = YES;
					cc_edge_curves_2[cccc_ind]++;
				}

				if(cc_curve_id == 0){
					write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);

					cc_edge_curve_1_array[cccc_ind][cc_edge_curves_1[cccc_ind]++] = cc_curve_id;
					output_cc_edge_curve = YES;
				}

				if(bb_ind == 0)cc_curve_id = -cc_curve_id;

				halving_cc = YES;
				if(aa_face_is -> edge_half[AACC] == NO){
					if(bb_ind == 0){
						if(pnt1 -> bb <= coord || pnt2 -> bb <= coord){    /* CAUTION: floating point number based decision */
							halving_cc = NO;

							weight = aa_face_is -> weight;
							if(pnt1 -> bb <= pnt2 -> bb){
								pnt3 = pnt1;
								weight = sqrt(1.0 - weight * weight);
							}
							else
								pnt3 = pnt2;
							pnt4 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);
						}
					}
					else{
						if(pnt1 -> bb >= coord || pnt2 -> bb >= coord){    /* CAUTION: floating point number based decision */
							halving_cc = NO;

							weight = aa_face_is -> weight;
							if(pnt1 -> bb <= pnt2 -> bb){
								pnt3 = pnt2;
								weight = sqrt(1.0 - weight * weight);
							}
							else
								pnt3 = pnt1;
							pnt4 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);
						}
					}
				}											
				
				if(halving_cc == NO){
					add_vec(point3, *pnt3, shift_vec);
					add_vec(point4, *pnt4, shift_vec);
					
					point.aa = center.aa;
					point.bb = point4.bb;
					point.cc = point3.cc;
					
					write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point3.x, point3.y, point3.z);
					write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point4.x, point4.y, point4.z);
					write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, vertex_id - 1, vertex_id, CURVE_RATE);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
					
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

					if(bb_ind == 0)
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
					else
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
											
					write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

					aa_side_curve_array[aa_side_curves++] = curve_id;
				}
				else{
#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to %s\n\n", string_cc);
#endif
					pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
					pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
									
					size = fabs(pnt1 -> bb - pnt2 -> bb);
					size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
					if((bb_ind == 0 && center.bb > par_corner[0].bb) || (bb_ind == 1 && center.bb < par_corner[1].bb)){          /* CAUTION */
#else
						if(center.bb * sign[bb_ind] > par_corner[bb_ind].bb * sign[bb_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to cc-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
#ifndef REUSE_POINTS
							copy_vec(point5, point1);
							copy_vec(point6, point2);
#endif
							if(bb_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
							}
							else{
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
							}

							point3.cc -= 4.0 * size;
							point4.cc -= 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
							point5.cc += 4.0 * size;
							point6.cc += 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
							point3.cc += 8.0 * size;
							point4.cc += 8.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
											
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = cur_id1;
							aa_side_curve_array[aa_side_curves++] = cur_id2;
						}
						else{

/* halving parallel to cc-edge */ 

							point1.aa = point2.aa = par_corner[aa_ind].aa;
							point1.bb = point2.bb = center.bb;
													
							point1.cc = center.cc - 2.0 * size;
							point2.cc = center.cc + 2.0 * size;
												
							if(bb_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
										
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
										
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
										
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
										
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
										
							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
											
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}

				if(bb_ind == 0)cc_curve_id = -cc_curve_id;

				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
		}
		else if(bb_count == 2 && cc_count == 1){   /* crossing bb-edge, touching cc-edge */
			if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
				aa_patch_id = get_aa_side_patch_id(aa_ind);

				halving_bb = NO;
				if(aa_face_is -> edge_half[AABB] == YES){

/* halving is relevant only if processing part of ellipse not touching cc-edge */

					if(cc_edge_is -> side != ref_position[cc_ind])halving_bb = YES;
				}

				if(inclusion -> corner == OUT){
					is_point = &(cc_edge_is -> point[0]);
					ver_id = is_point -> id;
									
					if(ver_id == 0){
						point.aa = par_corner[aa_ind].aa;
						point.bb = par_corner[bb_ind].bb;
						point.cc = is_point -> data.coord + shift_cc; 

						write_record("vertex %ld xyz "FMT3, ver_id = ++vertex_id, point.x, point.y, point.z);

						if(cc_edge_is -> side == ref_position[cc_ind]){
							write_record(" tile %ld\n", tile_id);
							cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point -> id = ver_id;
							cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point -> data);
							output_cc_edge_vertex = YES;
							cc_edge_curves_2[cccc_ind]++;
						}
						else
							write_record(" virtual\n");
					}

					is_point1 = &(bb_edge_is -> point[0]);
					ver_id1 = is_point1 -> id;
									
					is_point2 = &(bb_edge_is -> point[1]);
					ver_id2 = is_point2 -> id;

					if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
						if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
						point1.aa = point2.aa = par_corner[aa_ind].aa;
						point1.cc = point2.cc = par_corner[cc_ind].cc;

						point1.bb = is_point1 -> data.coord + shift_bb; 
						point2.bb = is_point2 -> data.coord + shift_bb; 

						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);

						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point2 -> id = ver_id2;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point2 -> data);
						output_bb_edge_vertex = YES;
						bb_edge_curves_2[bbbb_ind]++;
					}

					if(bb_curve_id == 0){
						write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);

						bb_edge_curve_1_array[bbbb_ind][bb_edge_curves_1[bbbb_ind]++] = bb_curve_id;
						output_bb_edge_curve = YES;
					}
									
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;

					if(halving_bb == YES){
#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_bb);
#endif

/* half-ellipse arc with both points "on" bb-edge is processed */

						pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
										
						size = fabs(pnt1 -> cc - pnt2 -> cc);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((cc_ind == 0 && center.cc > par_corner[0].cc) || (cc_ind == 1 && center.cc < par_corner[1].cc)){          /* CAUTION */
#else
							if(center.cc * sign[cc_ind] > par_corner[cc_ind].cc * sign[cc_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to bb-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(cc_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}
											
								point3.bb += 4.0 * size;
								point4.bb += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.bb -= 4.0 * size;
								point6.bb -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.bb -= 8.0 * size;
								point4.bb -= 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to bb-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.cc = point2.cc = center.cc;
											
								point1.bb = center.bb - 2.0 * size;
								point2.bb = center.bb + 2.0 * size;
											
								if(cc_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}

								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
											
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
												
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else{
						//pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ */
						add_vec(point2, *pnt2, shift_vec);
										
						point.aa = center.aa;
						point.bb = par_corner[bb_ind].bb;
						point.cc = point2.cc;
										
						weight = aa_face_is -> weight;
						if(bb_ind == cc_ind)weight = sqrt(1.0 - weight * weight);

						write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, point2.z);

						if(cc_edge_is -> side != ref_position[cc_ind]){           

/* elliptical arc with both points on bb-edge is processed */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id, bg_vertex_id, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);

							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id_minus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id_minus, CURVE_RATE, tile_id);
												
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
						}
						else{

/* two elliptical arcs sharing point on cc-edge are processed */

/* process elliptical arc closer to tile corner at first */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_plus = ++curve_id, ver_id, bg_vertex_id, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, weight);
										
							if(bb_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2, bg_curve_id_plus, tile_id);
										
							if(bb_ind == cc_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id = ++curve_id, ver_id, ver_id3, bg_curve_id_plus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id = ++curve_id, ver_id3, ver_id, bg_curve_id_plus, CURVE_RATE, tile_id);
											
/* then process elliptical arc further from tile corner */
									
							if(aa_face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
								fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
								error_message(GENERAL_ERROR, "Unimplemented situation");
#endif
#ifdef DEBUG
								if(aa_face_is -> cross_halving == BB)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
								if(aa_face_is -> cross_halving == CC){                  /* halving with respect to cc-extreme point */
									pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
									pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
													
									size = fabs(pnt1 -> cc - pnt2 -> cc);
									size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
													
									add_vec(point1, *pnt1, shift_vec);
									add_vec(point2, *pnt2, shift_vec);
									copy_vec(point3, point1);
									copy_vec(point4, point2);
#ifndef REUSE_POINTS
									copy_vec(point5, point1);
									copy_vec(point6, point2);
#endif		
									if(cc_ind == 0){
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
										ver_id3 = bg_vertex_id;
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
									}
									else{
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
										ver_id4 = bg_vertex_id;
									}
													
									point3.bb += 4.0 * size;
									point4.bb += 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
													
									if(bb_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
													
									if(cc_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
									point5.bb -= 4.0 * size;
									point6.bb -= 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
									point3.bb -= 8.0 * size;
									point4.bb -= 8.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
									if(bb_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

									if(cc_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
								}
#ifdef MIN_MAX_HALVING
								else if(aa_face_is -> cross_halving == MIN){
								}
								else if(aa_face_is -> cross_halving == MAX){
								}
#endif

/* note: patch is topologically the same no matter what halving is applied */

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cur_id1, cur_id2, cur_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id;
								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{
								write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id, bg_vertex_id, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

								if(bb_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);

								if(bb_ind == cc_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id4, ver_id, bg_curve_id_minus, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id, ver_id4, bg_curve_id_minus, CURVE_RATE, tile_id);

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, cur_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id;
								aa_side_curve_array[aa_side_curves++] = curve_id;
							}
						}
					}
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
				}
				else{   /* BND */
					is_point = &(bb_edge_is -> point[1 - bb_ind]);
					ver_id = is_point -> id;

					if(ver_id == 0){
						point.aa = par_corner[aa_ind].aa;
						point.bb = is_point -> data.coord + shift_bb; 
						point.cc = par_corner[cc_ind].cc;

						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);

						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point -> id = ver_id;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point -> data);
						output_bb_edge_vertex = YES;
					}

					if(bb_ind == 0){
						ver_id1 = corner_id;
						ver_id2 = ver_id;
					}
					else{
						ver_id1 = ver_id;
						ver_id2 = corner_id;
					}
				
					if(bb_curve_id == 0){
						write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);
										
						bb_edge_curve_1_array[bbbb_ind][bb_edge_curves_1[bbbb_ind]++] = bb_curve_id;
						output_bb_edge_curve = YES;
					}
				
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
				
					if(halving_bb == YES){
#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_bb);
#endif
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
										
						size = fabs(pnt1 -> cc - pnt2 -> cc);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((cc_ind == 0 && center.cc > par_corner[0].cc) || (cc_ind == 1 && center.cc < par_corner[1].cc)){          /* CAUTION */
#else
							if(center.cc * sign[cc_ind] > par_corner[cc_ind].cc * sign[cc_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to bb-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(cc_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}
											
								point3.bb += 4.0 * size;
								point4.bb += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.bb -= 4.0 * size;
								point6.bb -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.bb -= 8.0 * size;
								point4.bb -= 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to bb-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.cc = point2.cc = center.cc;
											
								point1.bb = center.bb - 2.0 * size;
								point2.bb = center.bb + 2.0 * size;
											
								if(cc_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}

								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
											
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
												
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else{
						weight = aa_face_is -> weight;
						if(bb_ind == cc_ind)weight = sqrt(1.0 - weight * weight);
										
						//pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ on cc-edge */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ */
						add_vec(point2, *pnt2, shift_vec);

						point.aa = center.aa;
						point.bb = par_corner[bb_ind].bb;
						point.cc = point2.cc;

						write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, point2.z);
						write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, corner_id, bg_vertex_id, CURVE_RATE);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

						if(bb_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

						if(cc_ind == 0){
							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id1, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id2, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
						}											
						else{
							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id1, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id2, bg_curve_id, CURVE_RATE, tile_id);
						}			
								
						write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

						aa_side_curve_array[aa_side_curves++] = curve_id;
					}
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
				}
				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
			else{
				if(inclusion -> corner == OUT){
					if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
						is_point = &(cc_edge_is -> point[0]);
						ver_id = is_point -> id;
										
						if(ver_id == 0){
							pnt = &(aa_face_is -> extreme_pnt[2 * AABB + 1 - bb_ind]);          /* bb- or bb+ on cc-edge */
							add_vec(point, *pnt, shift_vec);

							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);
											
							cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point -> id = ver_id;
							cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point -> data);
							output_cc_edge_vertex = YES;
							cc_edge_curves_2[cccc_ind]++;
						}
					}
				}
			}
		}
		else if(bb_count == 1 && cc_count == 2){   /* crossing cc-edge, touching bb-edge */
			if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
				aa_patch_id = get_aa_side_patch_id(aa_ind);

				halving_cc = NO;
				if(aa_face_is -> edge_half[AACC] == YES){

/* halving is relevant only if processing part of ellipse not touching bb-edge */

					if(bb_edge_is -> side != ref_position[bb_ind])halving_cc = YES;
				}

				if(inclusion -> corner == OUT){
					is_point = &(bb_edge_is -> point[0]);
					ver_id = is_point -> id;
									
					if(ver_id == 0){
						point.aa = par_corner[aa_ind].aa;
						point.bb = is_point -> data.coord + shift_bb; 
						point.cc = par_corner[cc_ind].cc;

						write_record("vertex %ld xyz "FMT3, ver_id = ++vertex_id, point.x, point.y, point.z);

						if(bb_edge_is -> side == ref_position[bb_ind]){
							write_record(" tile %ld\n", tile_id);
							bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point -> id = ver_id;
							bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point -> data);
							output_bb_edge_vertex = YES;
							bb_edge_curves_2[bbbb_ind]++;
						}
						else
							write_record(" virtual\n");
					}

					is_point1 = &(cc_edge_is -> point[0]);
					ver_id1 = is_point1 -> id;
									
					is_point2 = &(cc_edge_is -> point[1]);
					ver_id2 = is_point2 -> id;

					if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
						if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
						point1.aa = point2.aa = par_corner[aa_ind].aa;
						point1.bb = point2.bb = par_corner[bb_ind].bb;

						point1.cc = is_point1 -> data.coord + shift_cc; 
						point2.cc = is_point2 -> data.coord + shift_cc; 

						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);	


						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point1 -> id = ver_id1;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point1 -> data);
						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point2 -> id = ver_id2;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point2 -> data);
						output_cc_edge_vertex = YES;
						cc_edge_curves_2[cccc_ind]++;
					}

					if(cc_curve_id == 0){
						write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);

						cc_edge_curve_1_array[cccc_ind][cc_edge_curves_1[cccc_ind]++] = cc_curve_id;
						output_cc_edge_curve = YES;
					}

					if(bb_ind == 0)cc_curve_id = -cc_curve_id;

					if(halving_cc == YES){
#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_cc);
#endif

/* half-ellipse arc with both points "on" cc-edge is processed */

						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
							
						size = fabs(pnt1 -> bb - pnt2 -> bb);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((bb_ind == 0 && center.bb > par_corner[0].bb) || (bb_ind == 1 && center.bb < par_corner[1].bb)){          /* CAUTION */
#else
							if(center.bb * sign[bb_ind] > par_corner[bb_ind].bb * sign[bb_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to cc-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(bb_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}
											
								point3.cc -= 4.0 * size;
								point4.cc -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.cc += 4.0 * size;
								point6.cc += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.cc += 8.0 * size;
								point4.cc += 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, tile_id);

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to cc-edge */ 

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.bb = point2.bb = center.bb;
											
								point1.cc = center.cc - 2.0 * size;
								point2.cc = center.cc + 2.0 * size;
											
								if(bb_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}
											
								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
											
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
											
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else{
						//pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ on bb-edge */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ */
						add_vec(point2, *pnt2, shift_vec);
										
						point.aa = center.aa;
						point.bb = point2.bb;
						point.cc = par_corner[cc_ind].cc;

						weight = aa_face_is -> weight;
						if(cc_ind == bb_ind)weight = sqrt(1.0 - weight * weight);

						write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, point2.z);

						if(bb_edge_is -> side != ref_position[bb_ind]){

/* elliptical arc with both points on cc-edge is processed */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id, bg_vertex_id, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);

							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id_minus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id_minus, CURVE_RATE, tile_id);
												
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
						}
						else{

/* two elliptical arcs sharing point on bb-edge are processed */

/* process elliptical arc closer to tile corner at first */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_plus = ++curve_id, ver_id, bg_vertex_id, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, weight);
										
							if(cc_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2, bg_curve_id_plus, tile_id);
										
							if(bb_ind == cc_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id = ++curve_id, ver_id3, ver_id, bg_curve_id_plus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id = ++curve_id, ver_id, ver_id3, bg_curve_id_plus, CURVE_RATE, tile_id);
											
/* then process elliptical arc further from tile corner */
									
							if(aa_face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
								fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
								error_message(GENERAL_ERROR, "Unimplemented situation");
#endif
#ifdef DEBUG
								if(aa_face_is -> cross_halving == CC)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
								if(aa_face_is -> cross_halving == BB){                   /* halving with respect to bb-extreme point */
									pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
									pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
													
									size = fabs(pnt1 -> bb - pnt2 -> bb);
									size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
													
									add_vec(point1, *pnt1, shift_vec);
									add_vec(point2, *pnt2, shift_vec);
									copy_vec(point3, point1);
									copy_vec(point4, point2);
#ifndef REUSE_POINTS
									copy_vec(point5, point1);
									copy_vec(point6, point2);
#endif
									if(bb_ind == 0){
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
										ver_id3 = bg_vertex_id;
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
									}
									else{
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
										ver_id4 = bg_vertex_id;
									}
													
									point3.cc -= 4.0 * size;
									point4.cc -= 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
													
									if(cc_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

									if(bb_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
									point5.cc += 4.0 * size;
									point6.cc += 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
									point3.cc += 8.0 * size;
									point4.cc += 8.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
									if(cc_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
													
									if(bb_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
								}		
#ifdef MIN_MAX_HALVING
								else if(aa_face_is -> cross_halving == MIN){
								}
								else if(aa_face_is -> cross_halving == MAX){
								}
#endif

/* note: patch is topologically the same no matter what halving is applied */

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, cur_id1, cur_id2, cur_id, tile_id, hole_spec, mirror_spec, side_spec);
								
								aa_side_curve_array[aa_side_curves++] = cur_id;
								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{
								write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id, bg_vertex_id, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

								if(cc_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);

								if(bb_ind == cc_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id, ver_id3, bg_curve_id_minus, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id3, ver_id, bg_curve_id_minus, CURVE_RATE, tile_id);

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, cur_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id;
								aa_side_curve_array[aa_side_curves++] = curve_id;
							}
						}
					}
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
				}
				else{   /* BND */
					is_point = &(cc_edge_is -> point[1 - cc_ind]);
					ver_id = is_point -> id;

					if(ver_id == 0){
						point.aa = par_corner[aa_ind].aa;
						point.bb = par_corner[bb_ind].bb;
						point.cc = is_point -> data.coord + shift_cc; 

						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);

						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point -> id = ver_id;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point -> data);
						output_cc_edge_vertex = YES;
					}
									
					if(cc_ind == 0){
						ver_id1 = corner_id;
						ver_id2 = ver_id;
					}
					else{
						ver_id1 = ver_id;
						ver_id2 = corner_id;
					}
				
					if(cc_curve_id == 0){
						write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);

						cc_edge_curve_1_array[cccc_ind][cc_edge_curves_1[cccc_ind]++] = cc_curve_id;
						output_cc_edge_curve = YES;
					}

					if(bb_ind == 0)cc_curve_id = -cc_curve_id;

					if(halving_cc == YES){
#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_cc);
#endif
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
										
						size = fabs(pnt1 -> bb - pnt2 -> bb);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((bb_ind == 0 && center.bb > par_corner[0].bb) || (bb_ind == 1 && center.bb < par_corner[1].bb)){          /* CAUTION */
#else
							if(center.bb * sign[bb_ind] > par_corner[bb_ind].bb * sign[bb_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to cc-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(bb_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}												

								point3.cc -= 4.0 * size;
								point4.cc -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.cc += 4.0 * size;
								point6.cc += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.cc += 8.0 * size;
								point4.cc += 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
													
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to cc-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.bb = point2.bb = center.bb;
													
								point1.cc = center.cc - 2.0 * size;
								point2.cc = center.cc + 2.0 * size;
												
								if(bb_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}
												
								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
												
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
												
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
												
								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
													
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else{
						weight = aa_face_is -> weight;
						if(cc_ind == bb_ind)weight = sqrt(1.0 - weight * weight);

						//pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ on bb-edge */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ */
						add_vec(point2, *pnt2, shift_vec);

						point.aa = center.aa;
						point.bb = point2.bb;
						point.cc = par_corner[cc_ind].cc;

						write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, point2.z);
						write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, corner_id, bg_vertex_id, CURVE_RATE);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

						if(cc_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
				
						if(bb_ind == 0){
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id1, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id2, bg_curve_id, CURVE_RATE, tile_id);
						}
						else{
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id1, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id2, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
						}

						write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

						aa_side_curve_array[aa_side_curves++] = curve_id;
					}
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
				}
				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
			else{
				if(inclusion -> corner == OUT){
					if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
						is_point = &(bb_edge_is -> point[0]);
						ver_id = is_point -> id;
										
						if(ver_id == 0){
							pnt = &(aa_face_is -> extreme_pnt[2 * AACC + 1 - cc_ind]);          /* cc- or cc+ on bb-edge */
							add_vec(point, *pnt, shift_vec);

							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, point.z, tile_id);
											
							bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point -> id = ver_id;
							bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point -> data);
							output_bb_edge_vertex = YES;
							bb_edge_curves_2[bbbb_ind]++;
						}
					}
				}
			}
		}
		else if(bb_count == 2 && cc_count == 2){   /* crossing both bb-edge and cc-edge */
			if((bb_edge_is -> side & ref_position[bb_ind]) != 0 || (cc_edge_is -> side & ref_position[cc_ind]) != 0){
				aa_patch_id = get_aa_side_patch_id(aa_ind);
								
				halving_bb = halving_cc = NO;
				if(inclusion -> corner == OUT){
					if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
						is_point1 = &(bb_edge_is -> point[0]);
						ver_id1 = is_point1 -> id;
										
						is_point2 = &(bb_edge_is -> point[1]);
						ver_id2 = is_point2 -> id;
										
						if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
							if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
							point1.aa = point2.aa = par_corner[aa_ind].aa;
							point1.cc = point2.cc = par_corner[cc_ind].cc;
											
							point1.bb = is_point1 -> data.coord + shift_bb; 
							point2.bb = is_point2 -> data.coord + shift_bb; 

							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);

							bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
							bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
							bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point2 -> id = ver_id2;
							bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point2 -> data);
							output_bb_edge_vertex = YES;
							bb_edge_curves_2[bbbb_ind]++;
						}
										
						if(bb_curve_id == 0){
							write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id1, ver_id2, tile_id, output_spec);
											
							bb_edge_curve_1_array[bbbb_ind][bb_edge_curves_1[bbbb_ind]++] = bb_curve_id;
							output_bb_edge_curve = YES;
						}

						if(aa_face_is -> edge_half[AABB] == YES){

/* halving is relevant only if processing part of ellipse not intersecting cc-edge */

							if((cc_edge_is -> side & ref_position[cc_ind]) == 0)halving_bb = YES;
						}
					}
					if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
						is_point3 = &(cc_edge_is -> point[0]);
						ver_id3 = is_point3 -> id;
										
						is_point4 = &(cc_edge_is -> point[1]);
						ver_id4 = is_point4 -> id;
										
						if(ver_id3 == 0 || ver_id4 == 0){
#ifdef DEBUG
							if(ver_id3 != 0 || ver_id4 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
							point3.aa = point4.aa = par_corner[aa_ind].aa;
							point3.bb = point4.bb = par_corner[bb_ind].bb;

							point3.cc = is_point3 -> data.coord + shift_cc; 
							point4.cc = is_point4 -> data.coord + shift_cc; 
											
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point3.x, point3.y, point3.z, tile_id);
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point4.x, point4.y, point4.z, tile_id);

							cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point3 -> id = ver_id3;
							cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point3 -> data);
							cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point4 -> id = ver_id4;
							cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point4 -> data);
							output_cc_edge_vertex = YES;
							cc_edge_curves_2[cccc_ind]++;
						}
										
						if(cc_curve_id == 0){
							write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id3, ver_id4, tile_id, output_spec);
											
							cc_edge_curve_1_array[cccc_ind][cc_edge_curves_1[cccc_ind]++] = cc_curve_id;
							output_cc_edge_curve = YES;
						}

						if(aa_face_is -> edge_half[AACC] == YES){

/* halving is relevant only if processing part of ellipse not intersecting bb-edge */

							if((bb_edge_is -> side & ref_position[bb_ind]) == 0)halving_cc = YES;
						}
					}
									
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;

					if(halving_bb == YES){

/* half-ellipse arc with both points "on" bb-edge is processed */

#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_bb);
#endif
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
										
						size = fabs(pnt1 -> cc - pnt2 -> cc);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((cc_ind == 0 && center.cc > par_corner[0].cc) || (cc_ind == 1 && center.cc < par_corner[1].cc)){          /* CAUTION */
#else
							if(center.cc * sign[cc_ind] > par_corner[cc_ind].cc * sign[cc_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to bb-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(cc_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}
											
								point3.bb += 4.0 * size;
								point4.bb += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.bb -= 4.0 * size;
								point6.bb -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.bb -= 8.0 * size;
								point4.bb -= 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to bb-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.cc = point2.cc = center.cc;
											
								point1.bb = center.bb - 2.0 * size;
								point2.bb = center.bb + 2.0 * size;
											
								if(cc_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}

								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
											
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
												
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else if(halving_cc == YES){

/* half-ellipse arc with both points "on" cc-edge is processed */

#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_cc);
#endif
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
										
						size = fabs(pnt1 -> bb - pnt2 -> bb);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((bb_ind == 0 && center.bb > par_corner[0].bb) || (bb_ind == 1 && center.bb < par_corner[1].bb)){          /* CAUTION */
#else
							if(center.bb * sign[bb_ind] > par_corner[bb_ind].bb * sign[bb_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to cc-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(bb_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z);
								}

								point3.cc -= 4.0 * size;
								point4.cc -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id1, ver_id2, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id3, bg_curve_id, tile_id);

								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id2, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id1, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.cc += 4.0 * size;
								point6.cc += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id2, ver_id1, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.cc += 8.0 * size;
								point4.cc += 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id2, ver_id1, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id4, bg_curve_id, tile_id);

								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id2, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id1, bg_curve_id, CURVE_RATE, tile_id);
													
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to cc-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.bb = point2.bb = center.bb;
													
								point1.cc = center.cc - 2.0 * size;
								point2.cc = center.cc + 2.0 * size;
												
								if(bb_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}
												
								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
												
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z);
												
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id1, ver_id2, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id3, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id4, bg_curve_id, tile_id);
												
								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
													
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else{
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ */
											
						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
											
						point.aa = center.aa;
						point.bb = point1.bb;
						point.cc = point2.cc;
											
						weight = aa_face_is -> weight;
						if(bb_ind == cc_ind)weight = sqrt(1.0 - weight * weight);
											
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id9 = ++vertex_id, point1.x, point1.y, point1.z);
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id10 = ++vertex_id, point2.x, point2.y, point2.z);

						if(cc_edge_is -> side != ref_position[cc_ind]){

/* elliptical arc with both points on bb-edge is processed */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);

							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id_minus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id_minus, CURVE_RATE, tile_id);

							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
						}
						else if(bb_edge_is -> side != ref_position[bb_ind]){           

/* elliptical arc with both points on cc-edge is processed */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id3, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id4, bg_curve_id_minus, tile_id);

							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id7, ver_id8, bg_curve_id_minus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id8, ver_id7, bg_curve_id_minus, CURVE_RATE, tile_id);
												
							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
						}
						else{

/* two elliptical arcs joining points on bb-edge and cc-edge are processed */

/* process elliptical arc closer to tile corner at first */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_plus = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, weight);
											
							if(bb_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id_plus, tile_id);
											
							if(cc_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id3, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id4, bg_curve_id_plus, tile_id);
											
							if(bb_ind == cc_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id = ++curve_id, ver_id7, ver_id5, bg_curve_id_plus, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id = ++curve_id, ver_id5, ver_id7, bg_curve_id_plus, CURVE_RATE, tile_id);

/* then process elliptical arc further from tile corner */

							if(aa_face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
								fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
								error_message(GENERAL_ERROR, "Unimplemented situation");
#endif

/* then process elliptical arc further from tile corner */

								if(aa_face_is -> cross_halving == CC){                   /* halving with respect to cc-extreme point */
									pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
									pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
									
									size = fabs(pnt1 -> cc - pnt2 -> cc);
									size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */

									add_vec(point1, *pnt1, shift_vec);
									add_vec(point2, *pnt2, shift_vec);
									copy_vec(point3, point1);
									copy_vec(point4, point2);
#ifndef REUSE_POINTS
									copy_vec(point5, point1);
									copy_vec(point6, point2);
#endif		
									if(cc_ind == 0){
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z);
										ver_id5 = ver_id10;
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
									}
									else{
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z);
										ver_id6 = ver_id10;
									}
												
									point3.bb += 4.0 * size;
									point4.bb += 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id5, ver_id6, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
									if(bb_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else{
										if(cc_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3, bg_curve_id, tile_id);
									}
												
									if(cc_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
									point5.bb -= 4.0 * size;
									point6.bb -= 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
									point3.bb -= 8.0 * size;
									point4.bb -= 8.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
									if(bb_ind == 0){
										if(cc_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3, bg_curve_id, tile_id);
									}
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);

									if(cc_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id6, ver_id, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								}
								else if(aa_face_is -> cross_halving == BB){              /* halving with respect to bb-extreme point */
									pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
									pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
												
									size = fabs(pnt1 -> bb - pnt2 -> bb);
									size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
												
									add_vec(point1, *pnt1, shift_vec);
									add_vec(point2, *pnt2, shift_vec);
									copy_vec(point3, point1);
									copy_vec(point4, point2);
#ifndef REUSE_POINTS
									copy_vec(point5, point1);
									copy_vec(point6, point2);
#endif
									if(bb_ind == 0){
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z);
										ver_id5 = ver_id9;
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
									}
									else{
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z);
										ver_id6 = ver_id9;
									}
												
									point3.cc -= 4.0 * size;
									point4.cc -= 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id5, ver_id6, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
									if(cc_ind == 0){
										if(bb_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
									}
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3, bg_curve_id, tile_id);
												
									if(bb_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
									point5.cc += 4.0 * size;
									point6.cc += 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
									point3.cc += 8.0 * size;
									point4.cc += 8.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
									if(cc_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4, bg_curve_id, tile_id);
									else{
										if(bb_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
									}

									if(bb_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id6, ver_id, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								}		
#ifdef MIN_MAX_HALVING
								else if(aa_face_is -> cross_halving == MIN){
								}
								else if(aa_face_is -> cross_halving == MAX){
								}
#endif

/* note: patch is topologically the same no matter what halving is applied */

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cc_curve_id, cur_id1, cur_id2, cur_id, tile_id, hole_spec, mirror_spec, side_spec);
								
								aa_side_curve_array[aa_side_curves++] = cur_id;
								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{
								write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id_minus = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
										
/* IMPORTANT: fixed vertices are HERE not numbered increasingly along edge but increasingly from tile corner !!!
	 this will simplify curve forming */
										
								if(bb_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);

								if(cc_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id4, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id3, bg_curve_id_minus, tile_id);
								
								if(bb_ind == cc_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id6, ver_id8, bg_curve_id_minus, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id8, ver_id6, bg_curve_id_minus, CURVE_RATE, tile_id);
										
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cc_curve_id, curve_id, cur_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id;
								aa_side_curve_array[aa_side_curves++] = curve_id;
							}
						}
					}
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
				}
				else if(inclusion -> corner == BND){
					ver_id = corner_id;
									
					if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
						is_point1 = &(bb_edge_is -> point[1 - bb_ind]);
						ver_id1 = is_point1 -> id;
										
						if(ver_id1 == 0){
							point1.aa = par_corner[aa_ind].aa;
							point1.bb = is_point1 -> data.coord + shift_bb; 
							point1.cc = par_corner[cc_ind].cc;
											
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);

							bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
							bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
							output_bb_edge_vertex = YES;
						}
										
						if(bb_curve_id == 0){
							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id, ver_id1, tile_id, output_spec);
							else
								write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id1, ver_id, tile_id, output_spec);
											
							bb_edge_curve_1_array[bbbb_ind][bb_edge_curves_1[bbbb_ind]++] = bb_curve_id;
							output_bb_edge_curve = YES;
						}

						if(aa_face_is -> edge_half[AABB] == YES){

/* halving is relevant only if processing part of ellipse not intersecting cc-edge */

							if((cc_edge_is -> side & ref_position[cc_ind]) == 0)halving_bb = YES;
						}
					}
					if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
						is_point2 = &(cc_edge_is -> point[1 - cc_ind]);
						ver_id2 = is_point2 -> id;
										
						if(ver_id2 == 0){
							point2.aa = par_corner[aa_ind].aa;
							point2.bb = par_corner[bb_ind].bb;
							point2.cc = is_point2 -> data.coord + shift_cc; 
											
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);

							cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point2 -> id = ver_id2;
							cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point2 -> data);
							output_cc_edge_vertex = YES;
						}
										
						if(cc_curve_id == 0){
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id, ver_id2, tile_id, output_spec);
							else
								write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id2, ver_id, tile_id, output_spec);
											
							cc_edge_curve_1_array[cccc_ind][cc_edge_curves_1[cccc_ind]++] = cc_curve_id;
							output_cc_edge_curve = YES;
						}

						if(aa_face_is -> edge_half[AACC] == YES){

/* halving is relevant only if processing part of ellipse not intersecting bb-edge */

							if((bb_edge_is -> side & ref_position[bb_ind]) == 0)halving_cc = YES;
						}
					}
								
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
									
					if(halving_bb == YES){

/* half-ellipse arc with both points "on" bb-edge is processed */

#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_bb);
#endif
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
										
						size = fabs(pnt1 -> cc - pnt2 -> cc);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((cc_ind == 0 && center.cc > par_corner[0].cc) || (cc_ind == 1 && center.cc < par_corner[1].cc)){          /* CAUTION */
#else
							if(center.cc * sign[cc_ind] > par_corner[cc_ind].cc * sign[cc_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to bb-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif		
								if(cc_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}

								point3.bb += 4.0 * size;
								point4.bb += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
											
								if(bb_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);

								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.bb -= 4.0 * size;
								point6.bb -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.bb -= 8.0 * size;
								point4.bb -= 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								if(bb_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to bb-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.cc = point2.cc = center.cc;

								point1.bb = center.bb - 2.0 * size;
								point2.bb = center.bb + 2.0 * size;
												
								if(cc_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}
								
								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
								
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								
								if(bb_ind == 0){
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								}
								else{
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								}

								if(cc_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else if(halving_cc == YES){

/* half-ellipse arc with both points "on" cc-edge is processed */

#ifdef DEBUG_PRINT
						fprintf(stderr, "halving with respect to %s\n\n", string_cc);
#endif
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
										
						size = fabs(pnt1 -> bb - pnt2 -> bb);
						size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
#if 0
						if((bb_ind == 0 && center.bb > par_corner[0].bb) || (bb_ind == 1 && center.bb < par_corner[1].bb)){          /* CAUTION */
#else
							if(center.bb * sign[bb_ind] > par_corner[bb_ind].bb * sign[bb_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to cc-edge */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
#ifndef REUSE_POINTS
								copy_vec(point5, point1);
								copy_vec(point6, point2);
#endif
								if(bb_ind == 0){
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
								}
								else{
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
									write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								}												

								point3.cc -= 4.0 * size;
								point4.cc -= 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
								if(cc_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
								point5.cc += 4.0 * size;
								point6.cc += 4.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
								point3.cc += 8.0 * size;
								point4.cc += 8.0 * size;
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
								if(cc_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);

								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
													
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{

/* halving parallel to cc-edge */

								point1.aa = point2.aa = par_corner[aa_ind].aa;
								point1.bb = point2.bb = center.bb;
													
								point1.cc = center.cc - 2.0 * size;
								point2.cc = center.cc + 2.0 * size;
												
								if(bb_ind == 0){
									sub_vec(vec, *pnt2, *pnt1);
								}
								else{
									sub_vec(vec, *pnt1, *pnt2);
								}
								
								add_vec(point3, point1, vec);
								add_vec(point4, point2, vec);
								
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
								
								write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
								if(cc_ind == 0){
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								}
								else{
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								}
												
								if(bb_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
													
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

								aa_side_curve_array[aa_side_curves++] = curve_id;
#if 0 /* only to fix indentation */
							}
#else
						}
#endif
					}
					else{
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ */
											
						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
											
						point.aa = center.aa;
						point.bb = point1.bb;
						point.cc = point2.cc;
											
						weight = aa_face_is -> weight;
						if(bb_ind == cc_ind)weight = sqrt(1.0 - weight * weight);
											
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id9 = ++vertex_id, point1.x, point1.y, point1.z);
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id10 = ++vertex_id, point2.x, point2.y, point2.z);

						if(cc_edge_is -> side != ref_position[cc_ind]){

/* elliptical arc with both points on bb-edge is processed */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

							if(bb_ind == cc_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, tile_id);

							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
						}
						else if(bb_edge_is -> side != ref_position[bb_ind]){           

/* elliptical arc with both points on cc-edge is processed */

							write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
							
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

							if(bb_ind == cc_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g density 2 tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, tile_id);

							write_record("patch %ld normal %g %g %g boundary curve %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

							aa_side_curve_array[aa_side_curves++] = curve_id;
						}
						else{

/* elliptical arc joining points on bb-edge and cc-edge is processed */

							if(aa_face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
								fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
								error_message(GENERAL_ERROR, "Unimplemented situation");
#endif

/* then process elliptical arc further from tile corner */

								if(aa_face_is -> cross_halving == CC){                   /* halving with respect to cc-extreme point */
									pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
									pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
									
									size = fabs(pnt1 -> cc - pnt2 -> cc);
									size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */

									add_vec(point1, *pnt1, shift_vec);
									add_vec(point2, *pnt2, shift_vec);
									copy_vec(point3, point1);
									copy_vec(point4, point2);
#ifndef REUSE_POINTS
									copy_vec(point5, point1);
									copy_vec(point6, point2);
#endif		
									if(cc_ind == 0){
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z);
										ver_id5 = ver_id10;
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
									}
									else{
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z);
										ver_id6 = ver_id10;
									}
												
									point3.bb += 4.0 * size;
									point4.bb += 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id5, ver_id6, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
									if(bb_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
												
									if(cc_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
									point5.bb -= 4.0 * size;
									point6.bb -= 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
									point3.bb -= 8.0 * size;
									point4.bb -= 8.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
									if(bb_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
												
									if(cc_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id6, ver_id, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								}
								else if(aa_face_is -> cross_halving == BB){              /* halving with respect to bb-extreme point */
									pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
									pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
												
									size = fabs(pnt1 -> bb - pnt2 -> bb);
									size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */
												
									add_vec(point1, *pnt1, shift_vec);
									add_vec(point2, *pnt2, shift_vec);
									copy_vec(point3, point1);
									copy_vec(point4, point2);
#ifndef REUSE_POINTS
									copy_vec(point5, point1);
									copy_vec(point6, point2);
#endif
									if(bb_ind == 0){
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z);
										ver_id5 = ver_id9;
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
									}
									else{
										write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
										//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, point2.z);
										ver_id6 = ver_id9;
									}
												
									point3.cc -= 4.0 * size;
									point4.cc -= 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id5, ver_id6, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
									if(cc_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
												
									if(bb_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id5, ver_id, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
									point5.cc += 4.0 * size;
									point6.cc += 4.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
									point3.cc += 8.0 * size;
									point4.cc += 8.0 * size;
									write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id6, ver_id5, CURVE_RATE);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
									if(cc_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
												
									if(bb_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id6, ver_id, bg_curve_id, CURVE_RATE, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
								}		
#ifdef MIN_MAX_HALVING
								else if(aa_face_is -> cross_halving == MIN){
								}
								else if(aa_face_is -> cross_halving == MAX){
								}
#endif

/* note: patch is topologically the same no matter what halving is applied */

								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);
								
								aa_side_curve_array[aa_side_curves++] = cur_id1;
								aa_side_curve_array[aa_side_curves++] = cur_id2;
							}
							else{
								write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id9, ver_id10, CURVE_RATE);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
											
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

								if(bb_ind == cc_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
												
								write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);
								
								aa_side_curve_array[aa_side_curves++] = curve_id;
							}
						}
					}
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
				}
				else{  /* IN */
					ver_id = corner_id;
									
/* CAUTION: this is the case when intersection with all three edges occurs and each edge_is has side equal to BOTH;
	 this implies that is_points have coord already subjected to periodic repetition and only relevant tile shift has to be applied */
										
					is_point1 = &(bb_edge_is -> point[1 - bb_ind]);
					ver_id1 = is_point1 -> id;
									
					if(ver_id1 == 0){
						point1.aa = par_corner[aa_ind].aa;
						if(bb_ind == ind_bb)               /* <==> per_bb == 0 */
							point1.bb = is_point1 -> data.coord + shift_bb;               /* shift_bb does not contain periodic repetition */
						else                               /* <==> per_bb == 1 */
							point1.bb = is_point1 -> data.coord + shift_bb - sign[ind_bb] * tile.size.bb;    /* remove periodic repetition */
						point1.cc = par_corner[cc_ind].cc;
										
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);

						bb_edge_vertex_id_array[bbbb_ind][bb_edge_vertices[bbbb_ind]] = is_point1 -> id = ver_id1;
						bb_edge_vertex_data_array[bbbb_ind][bb_edge_vertices[bbbb_ind]++] = &(is_point1 -> data);
						output_bb_edge_vertex = YES;
					}
									
					if(bb_curve_id == 0){
						if(bb_ind == 0)
							write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id, ver_id1, tile_id, output_spec);
						else
							write_record("curve %ld vertex %ld %ld tile %ld %s\n", bb_curve_id = ++curve_id, ver_id1, ver_id, tile_id, output_spec);
										
						bb_edge_curve_1_array[bbbb_ind][bb_edge_curves_1[bbbb_ind]++] = bb_curve_id;
						output_bb_edge_curve = YES;
					}
									
					is_point2 = &(cc_edge_is -> point[1 - cc_ind]);
					ver_id2 = is_point2 -> id;
									
					if(ver_id2 == 0){
						point2.aa = par_corner[aa_ind].aa;
						point2.bb = par_corner[bb_ind].bb;
						if(cc_ind == ind_cc)               /* <==> per_cc == 0 */
							point2.cc = is_point2 -> data.coord + shift_cc;               /* shift_cc does not contain periodic repetition */
						else                               /* <==> per_cc == 1 */
							point2.cc = is_point2 -> data.coord + shift_cc - sign[ind_cc] * tile.size.cc;    /* remove periodic repetition */
										
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);

						cc_edge_vertex_id_array[cccc_ind][cc_edge_vertices[cccc_ind]] = is_point2 -> id = ver_id2;
						cc_edge_vertex_data_array[cccc_ind][cc_edge_vertices[cccc_ind]++] = &(is_point2 -> data);
						output_cc_edge_vertex = YES;
					}
									
					if(cc_curve_id == 0){
						if(cc_ind == 0)
							write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id, ver_id2, tile_id, output_spec);
						else
							write_record("curve %ld vertex %ld %ld tile %ld %s\n", cc_curve_id = ++curve_id, ver_id2, ver_id, tile_id, output_spec);
										
						cc_edge_curve_1_array[cccc_ind][cc_edge_curves_1[cccc_ind]++] = cc_curve_id;
						output_cc_edge_curve = YES;
					}
									
					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;

					if((cross_halving = aa_face_is -> cross_half) == YES){

/* cross halving is relevant only in particular quadrant;
	 note that I cannot use edge_is -> side as its value is BOTH for both edges;
	 therefore I have to check for coordinates of ellipse center with respect to tile corner;
	 although flouting point number based desicion is made, it should not be sensitive to round-off error 
	 because ellipse center is safely inside particular quadrant */

#if 0
						if((bb_ind == 0 && center.bb <= par_corner[0].bb) || (bb_ind == 1 && center.bb >= par_corner[1].bb))cross_halving = NO;
						if((cc_ind == 0 && center.cc <= par_corner[0].cc) || (cc_ind == 1 && center.cc >= par_corner[1].cc))cross_halving = NO;
#else
						if(center.bb * sign[bb_ind] <= par_corner[bb_ind].bb * sign[bb_ind])cross_halving = NO;
						if(center.cc * sign[cc_ind] <= par_corner[cc_ind].cc * sign[cc_ind])cross_halving = NO;
#endif
					}

					if(cross_halving == YES){
#ifdef DEBUG_PRINT												
						fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
						error_message(GENERAL_ERROR, "Unimplemented situation");
#endif
						if(aa_face_is -> cross_halving == CC){                   /* halving with respect to cc-extreme point */
							pnt1 = &(aa_face_is -> extreme_pnt[2 * AACC + 0]);     /* cc- */
							pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + 1]);     /* cc+ */
								
							size = fabs(pnt1 -> cc - pnt2 -> cc);
							size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
#ifndef REUSE_POINTS
							copy_vec(point5, point1);
							copy_vec(point6, point2);
#endif		
							if(cc_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
							}
							else{
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
							}
												
							point3.bb += 4.0 * size;
							point4.bb += 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
			
							if(bb_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id3, ver_id, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
							point5.bb -= 4.0 * size;
							point6.bb -= 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
							point3.bb -= 8.0 * size;
							point4.bb -= 8.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
							if(bb_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							
							if(cc_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
						}
						else if(aa_face_is -> cross_halving == BB){              /* halving with respect to bb-extreme point */
							pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + 0]);     /* bb- */
							pnt2 = &(aa_face_is -> extreme_pnt[2 * AABB + 1]);     /* bb+ */
											
							size = fabs(pnt1 -> bb - pnt2 -> bb);
							size = ellipse -> max * ellipse -> min / size;      /* constant area defined by associated diameters */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
#ifndef REUSE_POINTS
							copy_vec(point5, point1);
							copy_vec(point6, point2);
#endif
							if(bb_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z, tile_id);
							}
							else{
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
							}
											
							point3.cc -= 4.0 * size;
							point4.cc -= 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
												
							if(cc_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								
							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id, ver_id4, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id1 = ++curve_id, ver_id3, ver_id, bg_curve_id, CURVE_RATE, tile_id);
#ifndef REUSE_POINTS
							point5.cc += 4.0 * size;
							point6.cc += 4.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, point6.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, point5.z, 1.0 / 3.0);
#else
							point3.cc += 8.0 * size;
							point4.cc += 8.0 * size;
							write_record("curve %ld vertex %ld %ld order 4 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id4, ver_id3, CURVE_RATE);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, point4.z, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, point3.z, 1.0 / 3.0);
#endif
							if(cc_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							
							if(bb_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id4, ver_id, bg_curve_id, CURVE_RATE, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", cur_id2 = ++curve_id, ver_id, ver_id3, bg_curve_id, CURVE_RATE, tile_id);
						}
#ifdef MIN_MAX_HALVING
						else if(aa_face_is -> cross_halving == MIN){
						}
						else if(aa_face_is -> cross_halving == MAX){
						}
#endif

/* note: patch is topologically the same no matter what halving is applied */

						write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cc_curve_id, cur_id1, cur_id2, tile_id, hole_spec, mirror_spec, side_spec);
							
						aa_side_curve_array[aa_side_curves++] = cur_id1;
						aa_side_curve_array[aa_side_curves++] = cur_id2;
					}
					else{
						pnt1 = &(aa_face_is -> extreme_pnt[2 * AABB + bb_ind]);        /* bb- or bb+ */
						pnt2 = &(aa_face_is -> extreme_pnt[2 * AACC + cc_ind]);        /* cc- or cc+ */
									
						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
									
						point.aa = center.aa;
						point.bb = point1.bb;
						point.cc = point2.cc;
										
						weight = aa_face_is -> weight;
						if(bb_ind == cc_ind)weight = sqrt(1.0 - weight * weight);
										
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, point1.z);
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, point2.z);
						write_record("curve %ld vertex %ld %ld order 3 rate %g virtual\n", bg_curve_id = ++curve_id, ver_id3, ver_id4, CURVE_RATE);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, point.z, -weight);
										
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
										
						if(bb_ind == cc_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g tile %ld\n", ++curve_id, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, tile_id);
										
						write_record("patch %ld normal %g %g %g boundary curve %ld %ld %ld tile %ld %s %s %s\n", aa_patch_id, dir_aa.x, dir_aa.y, dir_aa.z, bb_curve_id, cc_curve_id, curve_id, tile_id, hole_spec, mirror_spec, side_spec);

						aa_side_curve_array[aa_side_curves++] = curve_id;
					}

					if(bb_ind == 0)cc_curve_id = -cc_curve_id;
					if(cc_ind == 1)bb_curve_id = -bb_curve_id;
				}
				aa_side_patch_array[aa_ind][aa_side_patches[aa_ind]++] = aa_patch_id;
#ifndef HALF_MAX_TILES
				aa_side_patches_total[aa_ind]++;
#endif
				output_patch = YES;
			}
		}
		else
			error_message(GENERAL_ERROR, "Unexpected situation");

		if(output_patch == YES){
			if(aa_side_patches[aa_ind] > max_side_patches[AA])error_message(GENERAL_ERROR, "Unexpected situation");
			if(aa_side_patches[aa_ind] > max_side_intersections){
				if(aa_ind == 0)
					error_message(GENERAL_ERROR, "Too many intersections with negative aa-side of particular tile %ld", tile.id);        /* caution: use tile.id */
				else
					error_message(GENERAL_ERROR, "Too many intersections with positive aa-side of particular tile %ld", tile.id);        /* caution: use tile.id */
			}
			//if(aa_patch_id < 0)error_message(GENERAL_ERROR, "Patch id overflow");
		}
		if(output_side_vertex == YES){
			if(aa_side_vertices[aa_ind] > max_side_vertices[AA])error_message(GENERAL_ERROR, "Unexpected situation");
		}
		if(output_bb_edge_vertex == YES){
			if(bb_edge_vertices[bbbb_ind] > max_edge_vertices[BB])error_message(GENERAL_ERROR, "Unexpected situation");
		}
		if(output_cc_edge_vertex == YES){
			if(cc_edge_vertices[cccc_ind] > max_edge_vertices[CC])error_message(GENERAL_ERROR, "Unexpected situation");
		}
		if(output_bb_edge_curve == YES){
			if(bb_edge_curves_1[bbbb_ind] > max_edge_curves_1[BB])error_message(GENERAL_ERROR, "Unexpected situation");
		}
		if(output_cc_edge_curve == YES){
			if(cc_edge_curves_1[cccc_ind] > max_edge_curves_1[CC])error_message(GENERAL_ERROR, "Unexpected situation");
		}
	}
}



/* this function replicates checks in process_aa_side_geometry */

static logic
check_inclusion_outside_tile_aa_side(void)
{
	ellipse_rec *ellipse = NULL;
	logic outside = YES;

	if((aa_face_is = inclusion -> face_is[AA]) != NULL){
		bb_edge_is = inclusion -> edge_is[BB];
		cc_edge_is = inclusion -> edge_is[CC];
		
		ellipse = &(aa_face_is -> ellipse);
		if(ellipse -> max == 0.0){                   /* side touching */
			if(bb_count == 0 && cc_count == 0){        /* not touching edge */
				if((aa_face_is -> side[AABB] & ref_position[bb_ind]) != 0 && (aa_face_is -> side[AACC] & ref_position[cc_ind]) != 0)outside = NO;
			}
			else{                                      /* touching edge */
				if(cc_count == 0){                       /* touching bb-edge */
					if((bb_edge_is -> side & ref_position[bb_ind]) != 0)outside = NO;
				}
				else if(bb_count == 0){                  /* touching cc-edge */
					if((cc_edge_is -> side & ref_position[cc_ind]) != 0)outside = NO;
				}
				else                                     /* touching corner */
					outside = NO;
			}
		}
		else{
			if(bb_count == 0 && cc_count == 0){        /* not crossing edges */
				if((aa_face_is -> side[AABB] & ref_position[bb_ind]) != 0 && (aa_face_is -> side[AACC] & ref_position[cc_ind]) != 0)outside = NO;
			}
			else if(bb_count == 1 && cc_count == 0){   /* touching bb-edge */
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0){
					if((aa_face_is -> side[AACC] & ref_position[cc_ind]) != 0)
						outside = NO;                        /* touching bb-edge from inside */
					else
						outside = NO;                        /* touching bb-edge from outside */
				}
			}
			else if(bb_count == 0 && cc_count == 1){   /* touching cc-edge */
				if((cc_edge_is -> side & ref_position[cc_ind]) != 0){
					if((aa_face_is -> side[AABB] & ref_position[bb_ind]) != 0)
						outside = NO;                        /* touching cc-edge from inside */
					else
						outside = NO;                        /* touching cc-edge from outside */
				}
			}
			else if(bb_count == 1 && cc_count == 1){   /* touching both bb-edge and cc-edge */
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0 && (cc_edge_is -> side & ref_position[cc_ind]) != 0)
					outside = NO;
				else{
					if((bb_edge_is -> side & ref_position[bb_ind]) != 0)outside = NO;
					if((cc_edge_is -> side & ref_position[cc_ind]) != 0)outside = NO;
				}
			}
			else if(bb_count == 2 && cc_count == 0){   /* crossing bb-edge */
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0)outside = NO;
			}
			else if(bb_count == 0 && cc_count == 2){   /* crossing cc-edge */
				if((cc_edge_is -> side & ref_position[cc_ind]) != 0)outside = NO;
			}
			else if(bb_count == 2 && cc_count == 1){   /* crossing bb-edge, touching cc-edge */
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0)
					outside = NO;
				else{
					if(inclusion -> corner == OUT){
						if((cc_edge_is -> side & ref_position[cc_ind]) != 0)outside = NO;
					}
				}
			}
			else if(bb_count == 1 && cc_count == 2){   /* crossing cc-edge, touching bb-edge */
				if((cc_edge_is -> side & ref_position[cc_ind]) != 0)
					outside = NO;
				else{
					if(inclusion -> corner == OUT){
						if((bb_edge_is -> side & ref_position[bb_ind]) != 0)outside = NO;
					}
				}
			}
			else if(bb_count == 2 && cc_count == 2){   /* crossing both bb-edge and cc-edge */
				if((bb_edge_is -> side & ref_position[bb_ind]) != 0 || (cc_edge_is -> side & ref_position[cc_ind]) != 0)outside = NO;
			}
			else
				error_message(GENERAL_ERROR, "Unexpected situation");
		}
	}

	return(outside);
}
